// TVTuner.h
//
// Requires DDK Only
// File created on 8/12/2008
//
#ifndef __TVTUNER_H__
#define __TVTUNER_H__

// Memory allocation pool tag
#define TVTUNER_POOL_TAG 'uTVT'

// Make all pool allocations tagged
#undef ExAllocatePool
#define ExAllocatePool(type, size) \
    ExAllocatePoolWithTag(type, size, TVTUNER_POOL_TAG);

// Number of driver managed queues used by this driver
#define NUMBER_OF_QUEUES    1

// queue start io callback
typedef VOID (*PTVTUNER_QUEUE_STARTIO)(
    IN PDEVICE_OBJECT   DeviceObject,
    IN PIRP             Irp 
    );

// irp queue type definition
typedef struct _TVTUNER_QUEUE
{
    PTVTUNER_QUEUE_STARTIO StartIoRoutine;
    PDEVICE_OBJECT  DeviceObject;
    LIST_ENTRY      IrpQueue;
    KSPIN_LOCK      QueueLock;
    PIRP            CurrentIrp;
    KEVENT          StopEvent;
    LONG            StallCount; 
    NTSTATUS        ErrorStatus;
    KDPC            TVTunerStartIoDpc;
    BOOLEAN         bUseTVTunerStartIoDpc;
} TVTUNER_QUEUE, *PTVTUNER_QUEUE;

// cancel-safe irp list type definition
typedef struct _TVTUNER_LIST
{
    PDEVICE_OBJECT  DeviceObject;
    LIST_ENTRY      IrpList;
    KSPIN_LOCK      ListLock;
    NTSTATUS        ErrorStatus;
} TVTUNER_LIST, *PTVTUNER_LIST;

// stall IRP list to syncronize Pnp, Power with
// the rest of IO
typedef struct _TVTUNER_IO_LOCK
{
    PDEVICE_OBJECT  DeviceObject;       // our device object
    KEVENT          StallCompleteEvent; // io stalled event
    LIST_ENTRY      StallIrpList;       // stalled irps
    KSPIN_LOCK      IoLock;             // spin lock to syncronize io with stall/unstall
    LONG            StallCount;         // number of times stall was requested
    LONG            ActiveIrpCount;     // number of oustanding, not-stalled IRPs
    NTSTATUS        ErrorStatus;
    PIRP            CurrentIrp;         // used by unstall code
} TVTUNER_IO_LOCK, *PTVTUNER_IO_LOCK;

// global (per driver) data block
typedef struct _TVTUNER_DATA
{
    UNICODE_STRING      RegistryPath;           // saved registry path
    ULONG               InstanceCount;          // FDO instance count
    USHORT              WdmVersion;             // os version
} TVTUNER_DATA, *PTVTUNER_DATA;

extern TVTUNER_DATA g_Data;

// PnP states
typedef enum _TVTUNER_PNP_STATE 
{
    PnpStateNotStarted = 0,
    PnpStateStarted,
    PnpStateStopPending,
    PnpStateStopped,
    PnpStateRemovePending,
    PnpStateRemoved,
    PnpStateSurpriseRemoved
} TVTUNER_PNP_STATE;


// bulk transfer context type
typedef struct _TVTUNER_IO_CONTEXT
{
    struct _TVTUNER_DEVICE_EXTENSION*   DeviceExtension;

    PUCHAR              NextVirtualAddress;
    ULONG               RemainingLength;
    ULONG               TotalTransferCount;
    PMDL                Mdl;
    PURB                Urb;
} TVTUNER_IO_CONTEXT, *PTVTUNER_IO_CONTEXT;

// The device extension for the device object
typedef struct _TVTUNER_DEVICE_EXTENSION
{
    PDEVICE_OBJECT          DeviceObject;           // pointer to the DeviceObject
    PDEVICE_OBJECT          PhysicalDeviceObject;   // underlying PDO
    PDEVICE_OBJECT          LowerDeviceObject;      // top of the device stack

    LONG                    RemoveCount;            // 1-based reference count
    KEVENT                  RemoveEvent;            // event to sync device removal

    TVTUNER_IO_LOCK IoLock;                 // misc io lock

    TVTUNER_PNP_STATE    PnpState;               // PnP state variable
    TVTUNER_PNP_STATE    PreviousPnpState;       // Previous PnP state variable

    UNICODE_STRING          InterfaceName;          // name returned from IoRegisterDeviceInterface

    SYSTEM_POWER_STATE      SystemPowerState;       // current system power state
    DEVICE_POWER_STATE      DevicePowerState;       // current device power state
    DEVICE_CAPABILITIES     DeviceCaps;             // device capabilities used to get S to D mappings
    PIRP                    SystemPowerIrp;         // Storage for system IRP during power transitions
    PIRP                    DevicePowerIrp;         // Storage for device IRP during power transitions
    PIO_WORKITEM            PowerWorkItem;          // Work item used for power transitions
    LONG                    bPowerStop;             // Flag to signal that power code stopped io
    LONG                    OpenHandleCount;

    PTVTUNER_QUEUE          QueueArray[NUMBER_OF_QUEUES];   // Array of our IRP queues
    TVTUNER_QUEUE           IOCTL_INTERNAL_USB_SUBMIT_URBQueue;                 // driver managed IRP queue


    PUSB_CONFIGURATION_DESCRIPTOR   ConfigDescriptor;   // configuration descriptor
    USBD_CONFIGURATION_HANDLE       ConfigHandle;       // configuration handle
    PUSBD_INTERFACE_INFORMATION     InterfaceInformation;
} TVTUNER_DEVICE_EXTENSION, *PTVTUNER_DEVICE_EXTENSION;

#ifdef __cplusplus
extern "C" 
{
#endif

NTSTATUS __stdcall DriverEntry(
    IN  PDRIVER_OBJECT  DriverObject,
    IN  PUNICODE_STRING RegistryPath
    );

NTSTATUS __stdcall TVTunerAddDevice(
    IN  PDRIVER_OBJECT  DriverObject,
    IN  PDEVICE_OBJECT  PhysicalDeviceObject
    );

VOID __stdcall TVTunerUnload(
    IN  PDRIVER_OBJECT  DriverObject
    );

NTSTATUS __stdcall TVTunerPnpDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall TVTunerPowerDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall TVTunerDeviceIoControlDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall TVTunerCreateDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall TVTunerCloseDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall TVTunerReadDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall TVTunerWriteDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall TVTunerCleanupDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall TVTunerSystemControlDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

///////////////////////////////////////////////////////////////////////////////////////////////////
// Queue Functions
///////////////////////////////////////////////////////////////////////////////////////////////////

VOID TVTunerInitializeQueue(
    IN  PTVTUNER_QUEUE    QueueExtension,
    IN  PTVTUNER_QUEUE_STARTIO     StartIoRoutine,
    IN  PDEVICE_OBJECT      DeviceObject,
    IN  BOOLEAN             bUseTVTunerStartIoDpc
    );

NTSTATUS TVTunerQueueIrp(
    IN  PTVTUNER_QUEUE    QueueExtension,
    IN  PIRP                Irp
    );

VOID TVTunerStartNext(
    IN  PTVTUNER_QUEUE    QueueExtension
    );

VOID TVTunerFlushQueue(
    IN  PTVTUNER_QUEUE    QueueExtension,
    IN  PFILE_OBJECT        FileObject
    );

VOID TVTunerInvalidateQueue(
    IN  PTVTUNER_QUEUE    Queue,
    IN  NTSTATUS                  ErrorStatus
    );

VOID TVTunerPauseQueue(
    IN  PTVTUNER_QUEUE    QueueExtension
    );
    
VOID TVTunerRestartQueue(
    IN  PTVTUNER_QUEUE    QueueExtension
    );

VOID TVTunerStartIoDpc(
    IN  PKDPC       Dpc,
    IN  PVOID       Context,
    IN  PVOID       Unused1,
    IN  PVOID       Unused2
    );

VOID TVTunerQueueCancelRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

///////////////////////////////////////////////////////////////////////////////////////////////////
// List functions
///////////////////////////////////////////////////////////////////////////////////////////////////

VOID TVTunerInitializeList(
    IN  PTVTUNER_LIST   List,
    IN  PDEVICE_OBJECT          DeviceObject
    );

NTSTATUS TVTunerInsertHead(
    IN  PTVTUNER_LIST   List, 
    IN  PIRP                Irp
    );

NTSTATUS TVTunerInsertTail(
    IN  PTVTUNER_LIST   List, 
    IN  PIRP                Irp
    );

PIRP TVTunerRemoveHead(
    IN  PTVTUNER_LIST   List
    );

PIRP TVTunerRemoveTail(
    IN  PTVTUNER_LIST   List
    );

VOID TVTunerFlushList(
    IN  PTVTUNER_LIST   List,
    IN  PFILE_OBJECT        FileObject
    );

VOID TVTunerInvalidateList(
    IN  PTVTUNER_LIST   List,
    IN  NTSTATUS                ErrorStatus
    );

VOID TVTunerListCancelRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

///////////////////////////////////////////////////////////////////////////////////////////////////
// TVTUNER_IO_LOCK
///////////////////////////////////////////////////////////////////////////////////////////////////

VOID TVTunerInitializeIoLock(
    IN  PTVTUNER_IO_LOCK    IoLock, 
    IN  PDEVICE_OBJECT              DeviceObject
    );

NTSTATUS TVTunerCheckIoLock(
    IN  PTVTUNER_IO_LOCK    IoLock, 
    IN  PIRP                        Irp
    );

VOID TVTunerPendingIoCancelRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS TVTunerIncrementIoCount(
    IN  PTVTUNER_IO_LOCK    IoLock
    );

VOID TVTunerDecrementIoCount(
    IN  PTVTUNER_IO_LOCK    IoLock
    );

VOID TVTunerLockIo(
    IN  PTVTUNER_IO_LOCK    IoLock
    );

VOID TVTunerWaitForStopIo(
    IN  PTVTUNER_IO_LOCK    IoLock
    );

VOID TVTunerUnlockIo(
    IN  PTVTUNER_IO_LOCK    IoLock
    );

VOID TVTunerFlushPendingIo(
    IN  PTVTUNER_IO_LOCK    IoLock,
    IN  PFILE_OBJECT                FileObject
    );

VOID TVTunerInvalidateIo(
    IN  PTVTUNER_IO_LOCK    IoLock,
    IN  NTSTATUS                    ErrorStatus
    );
#ifdef __cplusplus
}
#endif

BOOLEAN TVTunerIsStoppable(
    IN  PTVTUNER_DEVICE_EXTENSION    DeviceExtension
    );

BOOLEAN TVTunerIsRemovable(
    IN  PTVTUNER_DEVICE_EXTENSION    DeviceExtension
    );

BOOLEAN TVTunerAcquireRemoveLock(
    IN  PTVTUNER_DEVICE_EXTENSION    DeviceExtension
    );

VOID TVTunerReleaseRemoveLock(
    IN  PTVTUNER_DEVICE_EXTENSION    DeviceExtension
    );

VOID TVTunerWaitForSafeRemove(
    IN  PTVTUNER_DEVICE_EXTENSION    DeviceExtension
    );

NTSTATUS TVTunerSubmitIrpSync(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

VOID TVTunerStallQueues(
    IN  PTVTUNER_DEVICE_EXTENSION DeviceExtension
    );

VOID TVTunerRestartQueues(
    IN  PTVTUNER_DEVICE_EXTENSION DeviceExtension
    );

VOID TVTunerFlushQueues(
    IN  PTVTUNER_DEVICE_EXTENSION   DeviceExtension,
    IN  PFILE_OBJECT            FileObject
    );

///////////////////////////////////////////////////////////////////////////////////////////////////
// Pnp 
///////////////////////////////////////////////////////////////////////////////////////////////////

// define this PnP IRP.  This IRP is only defined in ntddk.h normally
#if !defined(IRP_MN_QUERY_LEGACY_BUS_INFORMATION)
#define IRP_MN_QUERY_LEGACY_BUS_INFORMATION     0x18
#endif // IRP_MN_QUERY_LEGACY_BUS_INFORMATION

NTSTATUS TVTunerStartDevice(
    IN  PTVTUNER_DEVICE_EXTENSION    DeviceExtension,
    IN  PIRP                            Irp
    );

NTSTATUS TVTunerFreeResources(
    IN  PTVTUNER_DEVICE_EXTENSION DeviceExtension
    );

NTSTATUS TVTunerGetDeviceCapabilities(
    IN  PTVTUNER_DEVICE_EXTENSION    DeviceExtension
    );

///////////////////////////////////////////////////////////////////////////////////////////////////
// Power
///////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _TVTUNER_POWER_IRP_CONTEXT
{
    PKEVENT         Event;
    NTSTATUS        Status;
} TVTUNER_POWER_IRP_CONTEXT, *PTVTUNER_POWER_IRP_CONTEXT;


#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

NTSTATUS TVTunerSystemPowerDispatch(
    IN  PTVTUNER_DEVICE_EXTENSION        DeviceExtension,
    IN  PIRP                        Irp
    );

NTSTATUS TVTunerSystemPowerCompletionRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp,
    IN  PVOID           Context
    );

NTSTATUS TVTunerDeviceQueryPowerCompletionRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp,
    IN  PVOID           Context
    );

NTSTATUS TVTunerDeviceSetPowerCompletionRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp,
    IN  PVOID           Context
    );

VOID TVTunerDevicePowerCompleteCallback(
    IN  PDEVICE_OBJECT      DeviceObject,
    IN  UCHAR               MinorFunction,
    IN  POWER_STATE         PowerState,
    IN  PVOID               Context,
    IN  PIO_STATUS_BLOCK    IoStatus
    );

#ifdef __cplusplus
}
#endif // __cplusplus

NTSTATUS TVTunerGetDevicePowerState(
    IN  PTVTUNER_DEVICE_EXTENSION        DeviceExtension,
    IN  SYSTEM_POWER_STATE              SystemState,
    OUT PDEVICE_POWER_STATE             DeviceState
    );

VOID TVTunerPowerDownPreparation(
    IN  PTVTUNER_DEVICE_EXTENSION        DeviceExtension,
    IN  PIO_WORKITEM_ROUTINE                Callback
    );

VOID TVTunerPowerDownPrepCallback(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PVOID           Context
    );

VOID TVTunerPowerD0PrepCallback(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PVOID           Context
    );

VOID TVTunerPowerUpCallback(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PVOID           Context
    );

VOID TVTunerRequestDevicePowerIrpCallback(
    IN  PDEVICE_OBJECT      DeviceObject,
    IN  UCHAR               MinorFunction,
    IN  POWER_STATE         PowerState,
    IN  PVOID               Context,
    IN  PIO_STATUS_BLOCK    IoStatus
    );

NTSTATUS TVTunerRequestDevicePowerIrp(
    IN  PTVTUNER_DEVICE_EXTENSION        DeviceExtension,
    IN  DEVICE_POWER_STATE             DeviceState
    );


///////////////////////////////////////////////////////////////////////////////////////////////////
// Registry
///////////////////////////////////////////////////////////////////////////////////////////////////

PVOID TVTunerRegQueryValueKey(
    IN  HANDLE  RegKeyHandle,
    IN  PWSTR   SubKeyName,
    IN  PWSTR   ValueName,
    OUT PULONG  Length
    );

VOID TVTunerRegEnumerateKeys(
    IN  HANDLE RegKeyHandle
    );

VOID TVTunerRegEnumerateValueKeys(
    IN  HANDLE  RegKeyHandle
    );

///////////////////////////////////////////////////////////////////////////////////////////////////
// Debug
///////////////////////////////////////////////////////////////////////////////////////////////////

// definition of debug levels

#define DBG_NONE            0
#define DBG_ERR             1
#define DBG_WARN            2
#define DBG_TRACE           3
#define DBG_INFO            4
#define DBG_VERB            5

#ifdef TVTUNER_WMI_TRACE

/*
tracepdb -f objchk_wxp_x86\i386\TVTuner.pdb -p C:\TVTuner
SET TRACE_FORMAT_SEARCH_PATH=C:\TVTuner

tracelog -start TVTuner -guid TVTuner.ctl -f TVTuner.log -flags 0x7FFFFFFF -level 5
tracelog -stop TVTuner

tracefmt -o TVTuner.txt -f TVTuner.log
*/

#define WPP_AREA_LEVEL_LOGGER(Area,Lvl)           WPP_LEVEL_LOGGER(Area)
#define WPP_AREA_LEVEL_ENABLED(Area,Lvl)          (WPP_LEVEL_ENABLED(Area) && WPP_CONTROL(WPP_BIT_##Area).Level >= Lvl)

#define WPP_CONTROL_GUIDS \
    WPP_DEFINE_CONTROL_GUID(TVTuner,(5138FF2E,690D,4450,A97D,612B0573C13C), \
        WPP_DEFINE_BIT(DBG_GENERAL)                 /* bit  0 = 0x00000001 */ \
        WPP_DEFINE_BIT(DBG_PNP)                     /* bit  1 = 0x00000002 */ \
        WPP_DEFINE_BIT(DBG_POWER)                   /* bit  2 = 0x00000004 */ \
        WPP_DEFINE_BIT(DBG_COUNT)                   /* bit  3 = 0x00000008 */ \
        WPP_DEFINE_BIT(DBG_CREATECLOSE)             /* bit  4 = 0x00000010 */ \
        WPP_DEFINE_BIT(DBG_WMI)                     /* bit  5 = 0x00000020 */ \
        WPP_DEFINE_BIT(DBG_UNLOAD)                  /* bit  6 = 0x00000040 */ \
        WPP_DEFINE_BIT(DBG_IO)                      /* bit  7 = 0x00000080 */ \
        WPP_DEFINE_BIT(DBG_INIT)                    /* bit  8 = 0x00000100 */ \
        WPP_DEFINE_BIT(DBG_09)                      /* bit  9 = 0x00000200 */ \
        WPP_DEFINE_BIT(DBG_10)                      /* bit 10 = 0x00000400 */ \
        WPP_DEFINE_BIT(DBG_11)                      /* bit 11 = 0x00000800 */ \
        WPP_DEFINE_BIT(DBG_12)                      /* bit 12 = 0x00001000 */ \
        WPP_DEFINE_BIT(DBG_13)                      /* bit 13 = 0x00002000 */ \
        WPP_DEFINE_BIT(DBG_14)                      /* bit 14 = 0x00004000 */ \
        WPP_DEFINE_BIT(DBG_15)                      /* bit 15 = 0x00008000 */ \
        WPP_DEFINE_BIT(DBG_16)                      /* bit 16 = 0x00010000 */ \
        WPP_DEFINE_BIT(DBG_17)                      /* bit 17 = 0x00020000 */ \
        WPP_DEFINE_BIT(DBG_18)                      /* bit 18 = 0x00040000 */ \
        WPP_DEFINE_BIT(DBG_19)                      /* bit 19 = 0x00080000 */ \
        WPP_DEFINE_BIT(DBG_20)                      /* bit 20 = 0x00100000 */ \
        WPP_DEFINE_BIT(DBG_21)                      /* bit 21 = 0x00200000 */ \
        WPP_DEFINE_BIT(DBG_22)                      /* bit 22 = 0x00400000 */ \
        WPP_DEFINE_BIT(DBG_23)                      /* bit 23 = 0x00800000 */ \
        WPP_DEFINE_BIT(DBG_24)                      /* bit 24 = 0x01000000 */ \
        WPP_DEFINE_BIT(DBG_25)                      /* bit 25 = 0x02000000 */ \
        WPP_DEFINE_BIT(DBG_26)                      /* bit 26 = 0x04000000 */ \
        WPP_DEFINE_BIT(DBG_27)                      /* bit 27 = 0x08000000 */ \
        WPP_DEFINE_BIT(DBG_28)                      /* bit 28 = 0x10000000 */ \
        WPP_DEFINE_BIT(DBG_29)                      /* bit 29 = 0x20000000 */ \
        WPP_DEFINE_BIT(DBG_30)                      /* bit 30 = 0x40000000 */ \
        WPP_DEFINE_BIT(DBG_31)                      /* bit 31 = 0x80000000 */ \
        )

__inline VOID TVTunerDumpIrp(
    IN PIRP Irp
    )
{
}

__inline PCHAR SystemPowerStateString(
    IN  SYSTEM_POWER_STATE  SystemState
    )
{
    return "";
}

__inline PCHAR DevicePowerStateString(
    IN  DEVICE_POWER_STATE  DeviceState
    )
{
    return "";
}

#else

// definition of debug areas

#define DBG_GENERAL         (1 << 0)
#define DBG_PNP             (1 << 1)
#define DBG_POWER           (1 << 2)
#define DBG_COUNT           (1 << 3)
#define DBG_CREATECLOSE     (1 << 4)
#define DBG_WMI             (1 << 5)
#define DBG_UNLOAD          (1 << 6)
#define DBG_IO              (1 << 7)
#define DBG_INIT            (1 << 8)

#define DBG_ALL             0xFFFFFFFF

#if DBG

VOID TVTunerDebugPrint(
    IN ULONG    Area,
    IN ULONG    Level,
    IN PCCHAR   Format,
    IN          ...
    );

VOID TVTunerDumpIrp(
    IN PIRP Irp
    );

PCHAR IrpMajorFunctionString(
    IN  UCHAR MajorFunction
    );

PCHAR PnPMinorFunctionString(
    IN  UCHAR   MinorFunction
    );

PCHAR PowerMinorFunctionString(
    IN  UCHAR   MinorFunction
    );

PCHAR SystemPowerStateString(
    IN  SYSTEM_POWER_STATE  SystemState
    );

PCHAR DevicePowerStateString(
    IN  DEVICE_POWER_STATE  DeviceState
    );

PCHAR WMIMinorFunctionString (
    IN  UCHAR MinorFunction
    );

#else	// !DBG

__inline VOID TVTunerDebugPrint(
    IN ULONG    Area,
    IN ULONG    Level,
    IN PCCHAR   Format,
    IN          ...
    )
{
}

#define TVTunerDumpIrp(Irp)
#define IrpMajorFunctionString(MajorFunction)
#define PnPMinorFunctionString(MinorFunction)
#define PowerMinorFunctionString(MinorFunction)
#define SystemPowerStateString(SystemState) 0
#define DevicePowerStateString(DeviceState) 0
#define WMIMinorFunctionString(MinorFunction)
#define TVTunerTdiDumpAddress(Address)

#endif	// DBG
#endif	// TVTUNER_WMI_TRACE

NTSTATUS TVTunerSubmitUrbSynch(
    IN  PTVTUNER_DEVICE_EXTENSION   DeviceExtension,
    IN  PURB                        Urb
    );

NTSTATUS TVTunerResetPipe(
    IN  PTVTUNER_DEVICE_EXTENSION   DeviceExtension,
    IN  USBD_PIPE_HANDLE                    PipeHandle
    );

NTSTATUS TVTunerAbortPipe(
    IN  PTVTUNER_DEVICE_EXTENSION   DeviceExtension,
    IN  USBD_PIPE_HANDLE                    PipeHandle
    );

NTSTATUS TVTunerResetDevice(
    IN  PTVTUNER_DEVICE_EXTENSION   DeviceExtension
    );

NTSTATUS TVTunerSelectAlternateInterface(
    IN  PTVTUNER_DEVICE_EXTENSION   DeviceExtension,
    IN  ULONG                               AlternateSetting
    );

NTSTATUS TVTunerPipe0IoCompletionRoutine(
    IN  PDEVICE_OBJECT DeviceObject,
    IN  PIRP           Irp,
    IN  PVOID          Context
    );

NTSTATUS TVTunerPipe0Io(
    IN  PTVTUNER_DEVICE_EXTENSION   DeviceExtension,
    IN  PIRP                                Irp,
    IN  PUSBD_PIPE_INFORMATION              PipeInformation
    );

NTSTATUS TVTunerPipe1IoCompletionRoutine(
    IN  PDEVICE_OBJECT DeviceObject,
    IN  PIRP           Irp,
    IN  PVOID          Context
    );

NTSTATUS TVTunerPipe1Io(
    IN  PTVTUNER_DEVICE_EXTENSION   DeviceExtension,
    IN  PIRP                                Irp,
    IN  PUSBD_PIPE_INFORMATION              PipeInformation
    );

#endif  // __TVTUNER_H__
