// wmi.c
//
// Requires DDK Only
// File created on 8/12/2008
//

#include "pch.h"
#ifdef TVTUNER_WMI_TRACE
#include "wmi.tmh"
#endif


///////////////////////////////////////////////////////////////////////////////////////////////////
//  TVTunerSystemControlDispatch
//      Dispatch routine for IRP_MJ_SYSTEM_CONTROL requests.
//
//  Arguments:
//      IN  DeviceObject
//              pointer to the device object for our device
//
//      IN  Irp
//              the IRP_MJ_SYSTEM_CONTROL IRP
//
//  Return Value:
//      NT status code.
//
NTSTATUS TVTunerSystemControlDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    )
{
    PTVTUNER_DEVICE_EXTENSION    deviceExtension;
    NTSTATUS                        status;
    PIO_STACK_LOCATION              stack;

    TVTunerDebugPrint(DBG_WMI, DBG_TRACE, __FUNCTION__"++. IRP %p", Irp);

    TVTunerDumpIrp(Irp);

    deviceExtension = (PTVTUNER_DEVICE_EXTENSION)DeviceObject->DeviceExtension;

    // protect againt IRP_MN_REMOVE_DEVICE
    if (!TVTunerAcquireRemoveLock(deviceExtension))
    {
        status = STATUS_DELETE_PENDING;

        Irp->IoStatus.Status = status;
        IoCompleteRequest (Irp, IO_NO_INCREMENT);

        TVTunerDebugPrint(DBG_WMI, DBG_WARN, __FUNCTION__"--. IRP %p, STATUS %x", Irp, status);

        return status;
    }

    // just pass it through
    IoSkipCurrentIrpStackLocation(Irp);
    status = IoCallDriver(deviceExtension->LowerDeviceObject, Irp);

    TVTunerReleaseRemoveLock(deviceExtension);

    TVTunerDebugPrint(DBG_WMI, DBG_TRACE, __FUNCTION__"--. IRP %p, STATUS %x", Irp, status);

    return status;
}
